@@ -20,7 +20,11 @@ package groovy.lang;
 
 import org.codehaus.groovy.reflection.ReflectionCache;
 import org.codehaus.groovy.reflection.stdclasses.CachedClosureClass;
-import org.codehaus.groovy.runtime.*;
+import org.codehaus.groovy.runtime.ComposedClosure;
+import org.codehaus.groovy.runtime.CurriedClosure;
+import org.codehaus.groovy.runtime.ExceptionUtils;
+import org.codehaus.groovy.runtime.InvokerHelper;
+import org.codehaus.groovy.runtime.InvokerInvocationException;
 import org.codehaus.groovy.runtime.callsite.BooleanClosureWrapper;
 import org.codehaus.groovy.runtime.memoize.LRUCache;
 import org.codehaus.groovy.runtime.memoize.Memoize;
@@ -66,7 +70,7 @@ public abstract class Closure<V> extends GroovyObjectSupport implements Cloneabl
      * With this resolveStrategy set the closure will attempt to resolve property references and methods to the
      * owner first, then the delegate (<b>this is the default strategy</b>).
      *
-     * For example the following code :
+     * For example the following code:
      * <pre>
      * class Test {
      *     def x = 30
@@ -77,21 +81,15 @@ public abstract class Closure<V> extends GroovyObjectSupport implements Cloneabl
      *         def cl = { y = x + y }
      *         cl.delegate = data
      *         cl()
-     *         println x
-     *         println y
-     *         println data
+     *         assert x == 30
+     *         assert y == 70
+     *         assert data == [x:10, y:20]
      *     }
      * }
      *
      * new Test().run()
      * </pre>
-     * will output :
-     * <pre>
-     * 30
-     * 70
-     * [x:10, y:20]
-     * </pre>
-     * because the x and y fields declared in the Test class shadow the variables in the delegate.<p>
+     * Will succeed, because the x and y fields declared in the Test class shadow the variables in the delegate.<p>
      * <i>Note that local variables are always looked up first, independently of the resolution strategy.</i>
      */
     public static final int OWNER_FIRST = 0;
@@ -100,8 +98,8 @@ public abstract class Closure<V> extends GroovyObjectSupport implements Cloneabl
      * With this resolveStrategy set the closure will attempt to resolve property references and methods to the
      * delegate first then the owner.
      *
-     * For example the following code :
-     * <pre>
+     * For example the following code:
+     * <pre class="groovyTestCase">
      * class Test {
      *     def x = 30
      *     def y = 40
@@ -112,21 +110,15 @@ public abstract class Closure<V> extends GroovyObjectSupport implements Cloneabl
      *         cl.delegate = data
      *         cl.resolveStrategy = Closure.DELEGATE_FIRST
      *         cl()
-     *         println x
-     *         println y
-     *         println data
+     *         assert x == 30
+     *         assert y == 40
+     *         assert data == [x:10, y:30]
      *     }
      * }
      *
      * new Test().run()
      * </pre>
-     * will output :
-     * <pre>
-     * 30
-     * 40
-     * [x:10, y:30]
-     * </pre>
-     * because the x and y variables declared in the delegate shadow the fields in the owner class.<p>
+     * This will succeed, because the x and y variables declared in the delegate shadow the fields in the owner class.<p>
      * <i>Note that local variables are always looked up first, independently of the resolution strategy.</i>
      */
     public static final int DELEGATE_FIRST = 1;
@@ -567,6 +559,10 @@ public abstract class Closure<V> extends GroovyObjectSupport implements Cloneabl
      * assert halver(8) == 4
      * </pre>
      *
+     * The position of the curried parameters will be calculated lazily, for example,
+     * if two overloaded doCall methods are available, the supplied arguments plus the
+     * curried arguments will be concatenated and the result used for method selection.
+     *
      * @param arguments the arguments to bind
      * @return the new closure with its arguments bound
      * @see #curry(Object...)
@@ -611,6 +607,10 @@ public abstract class Closure<V> extends GroovyObjectSupport implements Cloneabl
      * // [BEE, Cat, ant, dog]  Not found but would belong in position 3
      * </pre>
      *
+     * The position of the curried parameters will be calculated eagerly
+     * and implies all arguments prior to the specified n index are supplied.
+     * Default parameter values prior to the n index will not be available.
+     *
      * @param n the index from which to bind parameters (may be -ve in which case it will be normalized)
      * @param arguments the arguments to bind
      * @return the new closure with its arguments bound
