@@ -23,7 +23,6 @@ import groovy.lang.GroovyClassLoader;
 import groovy.lang.GroovyCodeSource;
 import groovy.lang.GroovyResourceLoader;
 import groovy.lang.Script;
-
 import org.codehaus.groovy.GroovyBugError;
 import org.codehaus.groovy.ast.ClassNode;
 import org.codehaus.groovy.ast.InnerClassNode;
@@ -70,9 +69,12 @@ import java.util.concurrent.ConcurrentHashMap;
  * @author Mattias Reichel
  */
 public class GroovyScriptEngine implements ResourceConnector {
+    private static final ClassLoader CL_STUB = AccessController.doPrivileged(new PrivilegedAction<ClassLoader>() {
+        public ClassLoader run() {
+            return new ClassLoader() {};
+        }
+    });
 
-    private static final ClassLoader CL_STUB = new ClassLoader() {
-    };
     private static final URL[] EMPTY_URL_ARRAY = new URL[0];
 
     private static class LocalData {
@@ -94,7 +96,7 @@ public class GroovyScriptEngine implements ResourceConnector {
     private URL[] roots;
     private ResourceConnector rc;
     private final ClassLoader parentLoader;
-    private final GroovyClassLoader groovyLoader;
+    private GroovyClassLoader groovyLoader;
     private final Map<String, ScriptCacheEntry> scriptCache = new ConcurrentHashMap<String, ScriptCacheEntry>();
     private CompilerConfiguration config;
 
@@ -226,14 +228,6 @@ public class GroovyScriptEngine implements ResourceConnector {
                 }
             });
 
-            final List<CompilationCustomizer> customizers = config.getCompilationCustomizers();
-            if (customizers != null) {
-                // GROOVY-4813 : apply configuration customizers
-                for (CompilationCustomizer customizer : customizers) {
-                    cu.addPhaseOperation(customizer, customizer.getPhase().getPhaseNumber());
-                }
-            }
-
             return cu;
         }
 
@@ -244,13 +238,25 @@ public class GroovyScriptEngine implements ResourceConnector {
             }
         }
 
-        private Class doParseClass(GroovyCodeSource codeSource) {
+        private Class<?> doParseClass(GroovyCodeSource codeSource) {
             // local is kept as hard reference to avoid garbage collection
             ThreadLocal<LocalData> localTh = getLocalData();
             LocalData localData = new LocalData();
             localTh.set(localData);
             StringSetMap cache = localData.dependencyCache;
+            Class<?> answer = null;
+            try {
+                updateLocalDependencyCache(codeSource, localData);
+                answer = super.parseClass(codeSource, false);
+                updateScriptCache(localData);
+            } finally {
+                cache.clear();
+                localTh.remove();
+            }
+            return answer;
+        }
 
+        private void updateLocalDependencyCache(GroovyCodeSource codeSource, LocalData localData) {
             // we put the old dependencies into local cache so createCompilationUnit
             // can pick it up. We put that entry under the name "."
             ScriptCacheEntry origEntry = scriptCache.get(codeSource.getName());
@@ -268,11 +274,13 @@ public class GroovyScriptEngine implements ResourceConnector {
 
                     }
                 }
+                StringSetMap cache = localData.dependencyCache;
                 cache.put(".", newDep);
             }
+        }
 
-            Class answer = super.parseClass(codeSource, false);
-
+        private void updateScriptCache(LocalData localData) {
+            StringSetMap cache = localData.dependencyCache;
             cache.makeTransitiveHull();
             long time = getCurrentTime();
             Set<String> entryNames = new HashSet<String>();
@@ -294,9 +302,6 @@ public class GroovyScriptEngine implements ResourceConnector {
                 ScriptCacheEntry cacheEntry = new ScriptCacheEntry(clazz, lastModified, time, value, false);
                 scriptCache.put(entryName, cacheEntry);
             }
-            cache.clear();
-            localTh.set(null);
-            return answer;
         }
 
         private String getPath(Class clazz, Map<String, String> precompiledEntries) {
@@ -355,15 +360,18 @@ public class GroovyScriptEngine implements ResourceConnector {
      * @return the parent classloader used to load scripts
      */
     private GroovyClassLoader initGroovyLoader() {
-        return (GroovyClassLoader) AccessController.doPrivileged(new PrivilegedAction() {
-            public Object run() {
-                if (parentLoader instanceof GroovyClassLoader) {
-                    return new ScriptClassLoader((GroovyClassLoader) parentLoader);
-                } else {
-                    return new ScriptClassLoader(parentLoader, config);
-                }
-            }
-        });
+        GroovyClassLoader groovyClassLoader =
+                AccessController.doPrivileged(new PrivilegedAction<ScriptClassLoader>() {
+                    public ScriptClassLoader run() {
+                        if (parentLoader instanceof GroovyClassLoader) {
+                            return new ScriptClassLoader((GroovyClassLoader) parentLoader);
+                        } else {
+                            return new ScriptClassLoader(parentLoader, config);
+                        }
+                    }
+                });
+        for (URL root : roots) groovyClassLoader.addURL(root);
+        return groovyClassLoader;
     }
 
     /**
@@ -473,7 +481,6 @@ public class GroovyScriptEngine implements ResourceConnector {
         if (parent == CL_STUB) parent = this.getClass().getClassLoader();
         this.parentLoader = parent;
         this.groovyLoader = initGroovyLoader();
-        for (URL root : roots) this.groovyLoader.addURL(root);
     }
 
     public GroovyScriptEngine(URL[] roots) {
@@ -492,7 +499,7 @@ public class GroovyScriptEngine implements ResourceConnector {
         if (urls == null) return null;
         URL[] roots = new URL[urls.length];
         for (int i = 0; i < roots.length; i++) {
-            if (urls[i].indexOf("://") != -1) {
+            if (urls[i].contains("://")) {
                 roots[i] = new URL(urls[i]);
             } else {
                 roots[i] = new File(urls[i]).toURI().toURL();
@@ -678,6 +685,7 @@ public class GroovyScriptEngine implements ResourceConnector {
     public void setConfig(CompilerConfiguration config) {
         if (config == null) throw new NullPointerException("configuration cannot be null");
         this.config = config;
+        this.groovyLoader = initGroovyLoader();
     }
 
     protected long getCurrentTime() {
