@@ -18,19 +18,18 @@
  */
 package org.codehaus.groovy.runtime.metaclass;
 
-import java.util.Map;
+import java.io.IOException;
+import java.io.Serializable;
+import java.util.AbstractCollection;
 import java.util.AbstractMap;
 import java.util.AbstractSet;
-import java.util.AbstractCollection;
-import java.util.Collection;
-import java.util.Set;
 import java.util.ArrayList;
-import java.util.Iterator;
+import java.util.Collection;
 import java.util.Enumeration;
+import java.util.Iterator;
+import java.util.Map;
 import java.util.NoSuchElementException;
-
-import java.io.Serializable;
-import java.io.IOException;
+import java.util.Set;
 
 /**
  * A hash table that supports mostly-concurrent reading, but
@@ -139,7 +138,8 @@ import java.io.IOException;
  */
 public class ConcurrentReaderHashMap 
   extends AbstractMap 
-  implements Map, Cloneable, Serializable {
+  implements Cloneable, Serializable {
+  private static final long serialVersionUID = -3225682440765612861L;
 
 
   /*
@@ -260,7 +260,7 @@ public class ConcurrentReaderHashMap
    * Returns the appropriate capacity (power of two) for the specified 
    * initial capacity argument.
    */
-  private int p2capacity(int initialCapacity) {
+  private static int p2capacity(int initialCapacity) {
     int cap = initialCapacity;
     
     // Compute the appropriate capacity
