@@ -70,6 +70,10 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 
+import static org.codehaus.groovy.ast.tools.GeneralUtils.args;
+import static org.codehaus.groovy.ast.tools.GeneralUtils.assignX;
+import static org.codehaus.groovy.ast.tools.GeneralUtils.callX;
+import static org.codehaus.groovy.ast.tools.GeneralUtils.classX;
 import static org.codehaus.groovy.ast.tools.GeneralUtils.returnS;
 import static org.codehaus.groovy.ast.tools.GeneralUtils.stmt;
 import static org.codehaus.groovy.ast.tools.GeneralUtils.varX;
@@ -214,34 +218,38 @@ public abstract class TraitComposer {
                     int fieldMods = 0;
                     int isStatic = 0;
                     boolean publicField = true;
-                    FieldNode helperField = fieldHelperClassNode.getField(Traits.FIELD_PREFIX + Traits.PUBLIC_FIELD_PREFIX + fieldName);
-                    if (helperField==null) {
-                        publicField = false;
-                        helperField = fieldHelperClassNode.getField(Traits.FIELD_PREFIX + Traits.PRIVATE_FIELD_PREFIX + fieldName);
+                    FieldNode helperField = null;
+                    fieldMods = 0;
+                    isStatic = 0;
+
+                    // look first for field with encoded modifier information
+                    for (Integer mod : Traits.FIELD_PREFIXES) {
+                        helperField = fieldHelperClassNode.getField(String.format("$0x%04x", mod) + fieldName);
+                        if (helperField != null) {
+                            if ((mod & Opcodes.ACC_STATIC) != 0) isStatic = Opcodes.ACC_STATIC;
+                            fieldMods = fieldMods | mod;
+                            break;
+                        }
                     }
-                    if (helperField==null) {
-                        publicField = true;
-                        // try to find a static one
-                        helperField = fieldHelperClassNode.getField(Traits.STATIC_FIELD_PREFIX+Traits.PUBLIC_FIELD_PREFIX+fieldName);
+
+                    if (helperField == null) {
+                        // look for possible legacy fields (trait compiled pre 2.4.8)
+                        helperField = fieldHelperClassNode.getField(Traits.FIELD_PREFIX + Traits.PUBLIC_FIELD_PREFIX + fieldName);
                         if (helperField==null) {
                             publicField = false;
-                            helperField = fieldHelperClassNode.getField(Traits.STATIC_FIELD_PREFIX+Traits.PRIVATE_FIELD_PREFIX +fieldName);
+                            helperField = fieldHelperClassNode.getField(Traits.FIELD_PREFIX + Traits.PRIVATE_FIELD_PREFIX + fieldName);
                         }
-                        fieldMods = fieldMods | Opcodes.ACC_STATIC;
-                        isStatic = Opcodes.ACC_STATIC;
-                    }
-                    if (helperField == null) {
-                        fieldMods = 0;
-                        isStatic = 0;
-                        for (Integer mod : Traits.FIELD_PREFIXES) {
-                            helperField = fieldHelperClassNode.getField(String.format("$0x%04x", mod) + fieldName);
-                            if (helperField != null) {
-                                if ((mod & Opcodes.ACC_STATIC) != 0) isStatic = Opcodes.ACC_STATIC;
-                                fieldMods = fieldMods | mod;
-                                break;
+                        if (helperField==null) {
+                            publicField = true;
+                            // try to find a static one
+                            helperField = fieldHelperClassNode.getField(Traits.STATIC_FIELD_PREFIX+Traits.PUBLIC_FIELD_PREFIX+fieldName);
+                            if (helperField==null) {
+                                publicField = false;
+                                helperField = fieldHelperClassNode.getField(Traits.STATIC_FIELD_PREFIX+Traits.PRIVATE_FIELD_PREFIX +fieldName);
                             }
+                            fieldMods = fieldMods | Opcodes.ACC_STATIC;
+                            isStatic = Opcodes.ACC_STATIC;
                         }
-                    } else {
                         fieldMods = fieldMods | (publicField?Opcodes.ACC_PUBLIC:Opcodes.ACC_PRIVATE);
                     }
                     if (getter) {
@@ -250,8 +258,20 @@ public abstract class TraitComposer {
                             List<AnnotationNode> copied = new LinkedList<AnnotationNode>();
                             List<AnnotationNode> notCopied = new LinkedList<AnnotationNode>();
                             GeneralUtils.copyAnnotatedNodeAnnotations(helperField, copied, notCopied);
-                            FieldNode fieldNode = cNode.addField(fieldName, fieldMods, returnType, (fieldMods & Opcodes.ACC_FINAL) == 0 ? null : helperField.getInitialExpression());
+                            FieldNode fieldNode = cNode.addField(fieldName, fieldMods, returnType, null);
                             fieldNode.addAnnotations(copied);
+                            // getInitialExpression above will be null if not in same source unit
+                            // so instead set within (static) initializer
+                            if (fieldNode.isFinal()) {
+                                String baseName = fieldNode.isStatic() ? Traits.STATIC_INIT_METHOD : Traits.INIT_METHOD;
+                                Expression mce = callX(helperClassNode, baseName + fieldNode.getName(), args(fieldNode.isStatic() ? classX(cNode) : varX("this")));
+                                Statement stmt = stmt(assignX(varX(fieldNode.getName(), fieldNode.getType()), mce));
+                                if (isStatic == 0) {
+                                    cNode.addObjectInitializerStatements(stmt);
+                                } else {
+                                    cNode.addStaticInitializerStatements(Collections.<Statement>singletonList(stmt), false);
+                                }
+                            }
                         }
                     }
                     Parameter[] newParams;
@@ -264,29 +284,30 @@ public abstract class TraitComposer {
                     }
 
                     Expression fieldExpr = varX(cNode.getField(fieldName));
+                    boolean finalSetter = !getter && (fieldMods & Opcodes.ACC_FINAL) != 0;
                     Statement body =
                             getter ? returnS(fieldExpr) :
-                                    stmt(
+                                    (finalSetter ? null : stmt(
                                             new BinaryExpression(
                                                     fieldExpr,
                                                     Token.newSymbol(Types.EQUAL, 0, 0),
                                                     varX(newParams[0])
                                             )
-                                    );
-                    if (getter || (fieldMods & Opcodes.ACC_FINAL) == 0) {
-                        MethodNode impl = new MethodNode(
-                                methodNode.getName(),
-                                Opcodes.ACC_PUBLIC | isStatic,
-                                returnType,
-                                newParams,
-                                ClassNode.EMPTY_ARRAY,
-                                body
-                        );
-                        AnnotationNode an = new AnnotationNode(COMPILESTATIC_CLASSNODE);
-                        impl.addAnnotation(an);
-                        cNode.addTransform(StaticCompileTransformation.class, an);
-                        cNode.addMethod(impl);
-                    }
+                                    ));
+                    // add getter/setter even though setter not strictly needed for final fields
+                    // but add empty body for setter for legacy compatibility
+                    MethodNode impl = new MethodNode(
+                            methodNode.getName(),
+                            Opcodes.ACC_PUBLIC | isStatic,
+                            returnType,
+                            newParams,
+                            ClassNode.EMPTY_ARRAY,
+                            body
+                    );
+                    AnnotationNode an = new AnnotationNode(COMPILESTATIC_CLASSNODE);
+                    impl.addAnnotation(an);
+                    cNode.addTransform(StaticCompileTransformation.class, an);
+                    cNode.addMethod(impl);
                 }
             }
         }
